//---------------------------------------------------------------------------
// File: C:\cbproj\Remote_CW_Keyer\Keyer_GUI.h
// Date: 2024-11-29
// Author: Wolfgang Buescher (DL4YHF)
// Purpose and revision history: See implementation, Keyer_GUI.cpp .


// Constants (as #defines) ..

#define EVT_MOUSE_DOWN 1 // .. for parameter iEvent in e.g. HandleMouseEventInTimingScope()
#define EVT_MOUSE_MOVE 2
#define EVT_MOUSE_UP   3


#define BANDLIST_BAND_COLUM_WIDTH  6  // .. measured in CHARACTERS, for the multi-column "frequency band list"
#define BANDLIST_FREQ_COLUMN_WIDTH 14 // worst case: " 1296.2345 USB"


// Variables ..

extern HINSTANCE APPL_hInstance; // handle to current instance, a Win32 API thing
extern BOOL APPL_fLaunchKeyerOnStart;
extern BOOL APPL_fPausedDuringSleep;
extern BOOL APPL_fSaveSettingsOnExit;

extern char g_sz255CommandLine[256];
extern char g_sz255PathToDataFiles[256];

// Colours for various GUI components that we'd like to control via SetColourScheme():
// (but convincing Windows and the VCL to actually USE these colours turned into
//  a nightmare; especially for stuff in the 'Non-Client Area', e.g. title and main menu)
extern TColor g_clMenuBackground, g_clMenuSelBackgnd, g_clMenuForeground;
extern TColor g_clDecoderOutputForeground, g_clDecoderOutputBackground;
extern int    g_iNewLanguage;
#define UI_MODE_SIMPLE   0 // only show the basic elements, menu items, etc in the UI
#define UI_MODE_COMPLETE 1 // show everything, including the stuff for debugging, SDR-like rig control, etc
extern int    g_iUserInterfaceMode;

extern volatile DWORD g_dwDebugMessage,  // kluge for debugging / dummy assignments for breakpoints..
               g_dwDebugWParam, g_dwDebugLParam;
extern volatile int   g_iDebugDummyIndex;
extern int g_iTimingScopeBitmapWidth;  // <- only for diagnostic / trouble with "destroyed forms"

#define KEYER_GUI_MAX_BANDS                  32
#define KEYER_GUI_MAX_STACKING_REGS_PER_BAND 3
extern int KeyerGUI_iBandStackingRegIndices[KEYER_GUI_MAX_BANDS][KEYER_GUI_MAX_STACKING_REGS_PER_BAND];
           // '--> filled in FillComboWithBandList(), used in CB_BandClick()


typedef struct t_ErrorFifoEntry
{
  char  sz511Text[512];
  int   iErrorClass;     // "error class" -> background colour in the RichText display
} T_ErrorFifoEntry;
# define C_ERROR_FIFO_SIZE 512

//----------------------------------------------------------------------------
// Global variables for hardcore debugging / post-mortem crash analysis.
//        Useful when the debugger's call stack shows nothing but garbage,
//        e.g. after an exception at the infamous address 0xFEEEFEEE .
//        ( C++Builder's debugger could still inspect SIMPLE GLOBAL
//          VARIABLES after most kinds of exceptions, but of course nothing
//          stack-based because the CPU- or task-stack was usually trashed.)
//----------------------------------------------------------------------------
#if(SWI_HARDCORE_DEBUGGING) // (1) = hardcore-debugging, (0)=normal compilation
extern int GUI_iLastSourceLine; // WATCH THIS after crashing with e.g. "0xFEEEFEEE"  ...
# define HERE_I_AM__GUI()  GUI_iLastSourceLine=__LINE__
# if( 0 )
    If certain worker threads of the application don't seem to terminate
    "politely", watch the following (complete list only HERE, in Keyer_Main.cpp):
     * GUI_iLastSourceLine
     * DSW_iLastSourceLine
     * DSP_iLastSourceLine
     * Keyer_iLastSourceLine
     * CwNet_iLastSourceLine
     * HttpSrv_iLastSourceLine
     * RigCtrl_iLastSourceLine
#  endif // (0) .. kludge to inspect the above variables in Borland via mouse-over ...
  extern int DSW_iLastSourceLine, DSP_iLastSourceLine, Keyer_iLastSourceLine,
             CwNet_iLastSourceLine, HttpSrv_iLastSourceLine;
  CPROT void CheckSystemHealth(const char *pszModuleName, int iSourceLine); // <- periodically called via macro CHECK_SYSTEM_HEALTH()
#else
# define HERE_I_AM__GUI()      /* rien */
#endif // SWI_HARDCORE_DEBUGGING ?


extern T_CwNet MyCwNet; // <- not a shiny C++ class, but a single instance representing the "CW Network"
extern DSoundWrapper MyDirectSound; // a "DirectSound wrapper" instance, first used for the SIDETONE OUTPUT

#if( SWI_USE_HAMLIB_SERVER ) // build an application with integrated "Hamlib Net rigctld"-compatible server ?
extern T_HLSrv MyHamlibServer; // Hamlib-'rigctld'-compatible server (guess the 'd' means 'daemon',
       // but neither demons nor daemons lurking in here. There's NOTHING running 'in the background'.)
#endif // SWI_USE_HAMLIB_SERVER ?


extern T_ErrorFifoEntry ErrorHistoryFifo[ C_ERROR_FIFO_SIZE ]; // <- classic lock-free circular FIFO between ShowError() and the GUI
extern int DEBUG_iErrorHistoryHead;
extern int DEBUG_iErrorHistoryTail;

typedef struct t_TimingScopeOverlay
{ int x1, y1, x2, y2; // mouse pointer coordinates at "mouse down" and "mouse up"
  int t1_ms;          // relative time in milliseconds (0=left edge) at "mouse down"
  int t2_ms;          // relative time in milliseconds (0=left edge) at "mouse up"
  BOOL visible;
} T_TimingScopeOverlay;
#define N_SCOPE_OVERLAYS 3 // allow showing up to THREE of these overlays simultaneously
extern T_TimingScopeOverlay TimingScopeOverlay[N_SCOPE_OVERLAYS];
extern int TimingScope_iCurrentOverlay;
extern int TimingScope_iUpdateCountOnTestTab, TimingScope_iUpdateCountOnKeyerTab;

extern T_SpecDispControl g_SpecDispControl;

extern const T_SL_TokenList C_TL_None_Zero[]; // list with only one item: "NONE".
extern const T_SL_TokenList SerialPortOutputSignals_Key[]; // "DTR", "RTS", optionally inverted
extern const T_SL_TokenList SerialPortOutputSignals_Radio[]; // similar as above, but on the "RADIO port"
extern const T_SL_TokenList SerialPortInputSignals_Key[]; // "DCD", "DSR", "CTS" and "RI"
extern const T_SL_TokenList SerialPortInputSignals_Radio[]; // similar as above, but on the "RADIO port"
extern const T_SL_TokenList RigControlMethods[]; // "No rig control", "CI-V", and who-knows-what
extern const T_SL_TokenList KeyTypes[]; // "None", "Straight", "Paddle", .. 
extern const T_SL_TokenList SidetonesOnTXD[]; // Sidetone frequencies available on a serial port's TXD(!) output
extern const T_SL_TokenList SidetonesOnAudioOut[]; // Sidetones sent to an audio device (with higher latency than "TXD sidetone"):
extern const T_SL_TokenList SidetoneRiseTimes[]; // "Rise"-, aka "Ramp"-times for sidetone VIA AUDIO OUTPUT
extern const char *CwKeyer_psz6MemoryDefaults[6];


//---------------------------------------------------------------------------
// Helper functions ( no shiny VCL class methods ! )
//    Details only in the implementation (Keyer_GUI.cpp) .
//---------------------------------------------------------------------------

void ParseCommandLine(const char *pszCmdLine);
void LoadSettings(void); // .. from an old-fashioned *.ini file, NOT from the bloody registry !
void SaveSettings(void); // .. in an old-fashioned *.ini file, NOT the bloody registry !
DWORD CalcHashForSettings(void); // .. to tell if we need to SAVE SETTINGS on exit
extern "C" void ShowError( int iErrorClass, const char * pszFormat, ... );
  // ,---------------------------'
  // '--> ERROR_CLASS_FATAL, ERROR_CLASS_ERROR, ERROR_CLASS_WARNING, ERROR_CLASS_INFO .
  //      If an optional 'debug run log' (file) is supported,
  //      iErrorClass bitmask SHOW_ERROR_IN_RUN_LOG can be used to emit
  //      the same message in "debug_run_log.txt", besides the GUI's "Debug" tab.
  //

//---------------------------------------------------------------------------
// Functions doomed to use Borland-specific data types like "TComboBox", etc:
//---------------------------------------------------------------------------

BOOL FillComboWithSerialPorts( TComboBox *pCombo, int iSelectedComPortNr );
BOOL FillComboWithItemsFromTokenLists( TComboBox *pCombo,
        const T_SL_TokenList *pTokens1, // [in] e.g. C_TL_None_Zero (with string = "None", value = zero)
        const T_SL_TokenList *pTokens2, // [in] e.g. SerialPortInputSignals_Key
        const T_SL_TokenList *pTokens3, // [in] e.g. SerialPortInputSignals_Radio, or NULL for only *one* list
        int iSelectedTokenValue  );     // [in] e.g. [-]KEYER_SIGNAL_INDEX_MORSE_KEY_CTS
BOOL FillComboWithBandList( TComboBox *pCombo, // [out] combo box with bands or even band-stacking registers
        DWORD dwSupportedBands, // [in] bitwise combination of RIGCTL_BAND_...
        DWORD dwSelectedBand ); // [in] a single bit representing the SELECTED band,
                                //      e.g. RIGCTL_BAND_40M .
int GetSelectedTokenValueFromComboBox( TComboBox *pCombo,
        const T_SL_TokenList *pTokens1, // [in] e.g. C_TL_None_Zero (with string = "None", value = zero)
        const T_SL_TokenList *pTokens2, // [in] e.g. SerialPortInputSignals_Key
        const T_SL_TokenList *pTokens3); // [in] e.g. SerialPortInputSignals_Radio, or NULL for only *one* list
  // If the selected text in the combo list doesn't match ANY of the
  // strings in the three token lists, this function returns
  // GUI_ERROR_NO_MATCHING_ENTRY to let THE CALLER decide what to use.
BOOL SelectComboItemByDecimalValue( TComboBox *pCombo, int iValue );
BOOL SelectComboItemByHexadecimalValue( TComboBox *pCombo, int iValue );
int  GetSerialPortSignalIndexFromComboBox( TComboBox *pCombo );
int ParseSerialPortNumberFromText( const char *pszSource );
double GetFrequencyFromEditField( TEdit *pEd, double *pdblFreqIncrement );
void UpdateTimingScope( T_KeyerTimingScope *pScope, Graphics::TBitmap *pDestBitmap);
void HandleMouseEventInTimingScope( Graphics::TBitmap *pBitmap, int iEvent, int x, int y );
void ClearScopeOverlays(void);
void SetColourScheme( int iColourScheme );
void UpdateSMeter( TImage *pImg, double dblSMeterLevel_dB );
      // Keep it simple, begin at "S zero" :
#     define C_SMETER_LEVEL_S0   0 /* "S0" on HF:       -20 dBuV */
#     define C_SMETER_LEVEL_S9  54 /* "S9" on HF:       +34 dBuV */
#     define C_SMETER_LEVEL_MAX 99 /* "S9+45 dB"                 */
void UpdateMultiFunctionMeter( // renders any of the horizontal bargraphs on the "Mult-funtion meter" (panel)
        TImage *pImg,  // [in,out] Borland-VCL-style "graphic image", with a "canvas" to paint into
        int x1, int y1, int x2, int y2, // [in] graphic area for one of the six(?) "meters" within pImg
        double dblValue, // [in] value to be displayed (percent, "dB", SWR, current, voltage, power, temperature..)
        double dblMinValue, double dblMaxValue, double dblStepwidth, // [in] physical value range (also used for the labelled scale)
        const char *pszParamName, // [in] name like "Power", "ALC", "COMP", "SWR", "Id", "Vd", "Temp"
        const char *pszPhysUnit,  // [in] physical unit (shown at the end of the tick scale), e.g. "%", "dB", "V", "A", "W", "°C"
        double dblRedStartValue, double dblRedEndValue ); // e.g. 10.0 .. 12.0 [Volts for the "red line" indicating UNDERVOLTAGE]
const char* RigCtrl_GetRadioControlPortAsString(void); // -> e.g. "COM5", if that's the port "talking to the radio" .
  // Called from within RigControl.c to emit 'info messages' like
  //     "RigControl: IC-9700 on COM5, 432250.0 kHz, CWN"  .
  // Since RighControl.c doesn't know anything about serial ports,
  // RigCtrl_GetRadioControlPortAsString() must be provided by the application.

/* EOF <Keyer_GUI.h> */